@@ -139,7 +139,8 @@ def record(seconds=3, sample_rate=DEFAULT_SAMPLE_RATE, normalize_db=0.1):
 
 def audio_bytes_to_np(wav_data,
                       sample_rate=DEFAULT_SAMPLE_RATE,
-                      normalize_db=0.1):
+                      normalize_db=0.1,
+                      mono=True):
   """Convert audio file data (in bytes) into a numpy array using Pydub.
 
   Args:
@@ -147,12 +148,15 @@ def audio_bytes_to_np(wav_data,
     sample_rate: Resample recorded audio to this sample rate.
     normalize_db: Normalize the audio to this many decibels. Set to None to skip
       normalization step.
+    mono: Force stereo signals to single channel. If false, output can one or
+      two channels depending on the source signal.
 
   Returns:
-    An array of the recorded audio at sample_rate.
+    An array of the recorded audio at sample_rate, shape [channels, time].
   """
   return note_seq.audio_io.wav_data_to_samples_pydub(
-      wav_data=wav_data, sample_rate=sample_rate, normalize_db=normalize_db)
+      wav_data=wav_data, sample_rate=sample_rate, normalize_db=normalize_db,
+      num_channels=1 if mono else None)
 
 
 def upload(sample_rate=DEFAULT_SAMPLE_RATE, normalize_db=None):
