@@ -15,7 +15,6 @@
 # Lint as: python3
 """Library of functions for differentiable digital signal processing (DDSP)."""
 
-
 import collections
 import copy
 from typing import Any, Dict, Optional, Sequence, Text, TypeVar
@@ -208,10 +207,24 @@ def gradient_reversal(x):
 
 
 # Unit Conversions -------------------------------------------------------------
-def midi_to_hz(notes: Number) -> Number:
-  """TF-compatible midi_to_hz function."""
+def midi_to_hz(notes: Number, midi_zero_silence: bool = False) -> Number:
+  """TF-compatible midi_to_hz function.
+
+  Args:
+    notes: Tensor containing encoded pitch in MIDI scale.
+    midi_zero_silence: Whether to output 0 hz for midi 0, which would be
+      convenient when midi 0 represents silence. By defualt (False), midi 0.0
+      corresponds to 8.18 Hz.
+
+  Returns:
+    hz: Frequency of MIDI in hz, same shape as input.
+  """
   notes = tf_float32(notes)
-  return 440.0 * (2.0**((notes - 69.0) / 12.0))
+  hz = 440.0 * (2.0 ** ((notes - 69.0) / 12.0))
+  # Map MIDI 0 as 0 hz when MIDI 0 is silence.
+  if midi_zero_silence:
+    hz = tf.where(tf.equal(notes, 0.0), 0.0, hz)
+  return hz
 
 
 def hz_to_midi(frequencies: Number) -> Number:
@@ -631,6 +644,41 @@ def upsample_with_windows(inputs: tf.Tensor,
   return x[:, hop_size:-hop_size, :]
 
 
+# TODO(jesseengel): Axis param, don't assume axis=1.
+def center_pad(audio, frame_size, mode='CONSTANT'):
+  """Pad an audio signal such that timestamps align to the center of frames.
+
+  Without centering, timestamps align to the front of frames.
+  Args:
+    audio: Input, shape [batch, time, ...].
+    frame_size: Size of each frame.
+    mode: Padding mode for tf.pad. One of "CONSTANT", "REFLECT", or
+      "SYMMETRIC" (case-insensitive).
+
+  Returns:
+    audio_padded: Shape [batch, time + (frame_size // 2) * 2, ...].
+  """
+  pad_amount = int(frame_size // 2)  # Symmetric even padding like librosa.
+  pads = [[0, 0] for _ in range(len(audio.shape))]
+  pads[1] = [pad_amount, pad_amount]
+  return tf.pad(audio, pads, mode=mode)
+
+
+def center_crop(audio, frame_size):
+  """Remove padding introduced from centering frames.
+
+  Inverse of center_pad().
+  Args:
+    audio: Input, shape [batch, time, ...].
+    frame_size: Size of each frame.
+
+  Returns:
+    audio_cropped: Shape [batch, time - (frame_size // 2) * 2, ...].
+  """
+  pad_amount = int(frame_size // 2)  # Symmetric even padding like librosa.
+  return audio[:, pad_amount:-pad_amount]
+
+
 # Synth conversions ------------------------------------------------------------
 def sinusoidal_to_harmonic(sin_amps,
                            sin_freqs,
@@ -793,6 +841,22 @@ def remove_above_nyquist(frequency_envelopes: tf.Tensor,
   return amplitude_envelopes
 
 
+def normalize_harmonics(harmonic_distribution, f0_hz=None, sample_rate=None):
+  """Normalize the harmonic distribution, optionally removing above nyquist."""
+  # Bandlimit the harmonic distribution.
+  if sample_rate is not None and f0_hz is not None:
+    n_harmonics = int(harmonic_distribution.shape[-1])
+    harmonic_frequencies = get_harmonic_frequencies(f0_hz, n_harmonics)
+    harmonic_distribution = remove_above_nyquist(
+        harmonic_frequencies, harmonic_distribution, sample_rate)
+
+  # Normalize
+  harmonic_distribution = safe_divide(
+      harmonic_distribution,
+      tf.reduce_sum(harmonic_distribution, axis=-1, keepdims=True))
+  return harmonic_distribution
+
+
 # TODO(jesseengel): Remove reliance on global injection for angular cumsum.
 @gin.configurable
 def oscillator_bank(frequency_envelopes: tf.Tensor,
@@ -848,6 +912,69 @@ def oscillator_bank(frequency_envelopes: tf.Tensor,
   return audio
 
 
+# TODO(jesseengel): Remove reliance on global injection for angular cumsum.
+@gin.configurable
+def harmonic_oscillator_bank(
+    frequency: tf.Tensor,
+    amplitude_envelopes: tf.Tensor,
+    initial_phase: Optional[tf.Tensor] = None,
+    sample_rate: int = 16000,
+    use_angular_cumsum: bool = True) -> tf.Tensor:
+  """Special oscillator bank for harmonic frequencies and streaming synthesis.
+
+
+  Args:
+    frequency: Sample-wise oscillator frequencies (Hz). Shape
+      [batch_size, n_samples, 1].
+    amplitude_envelopes: Sample-wise oscillator amplitude. Shape [batch_size,
+      n_samples, n_sinusoids].
+    initial_phase: Starting phase. Shape [batch_size, 1, 1].
+    sample_rate: Sample rate in samples per a second.
+    use_angular_cumsum: If synthesized examples are longer than ~100k audio
+      samples, consider use_angular_cumsum to avoid accumulating noticible phase
+      errors due to the limited precision of tf.cumsum. Unlike the rest of the
+      library, this property can be set with global dependency injection with
+      gin. Set the gin parameter `oscillator_bank.use_angular_cumsum=True`
+      to activate. Avoids accumulation of errors for generation, but don't use
+      usually for training because it is slower on accelerators.
+
+  Returns:
+    wav: Sample-wise audio. Shape [batch_size, n_samples, n_sinusoids] if
+      sum_sinusoids=False, else shape is [batch_size, n_samples].
+  """
+  frequency = tf_float32(frequency)
+  amplitude_envelopes = tf_float32(amplitude_envelopes)
+
+  # Angular frequency, Hz -> radians per sample.
+  omega = frequency * (2.0 * np.pi)  # rad / sec
+  omega = omega / float(sample_rate)  # rad / sample
+
+  # Accumulate phase and synthesize.
+  if use_angular_cumsum:
+    # Avoids accumulation errors.
+    phases = angular_cumsum(omega)
+  else:
+    phases = tf.cumsum(omega, axis=1)
+
+  if initial_phase is None:
+    initial_phase = tf.zeros([phases.shape[0], 1, 1])
+
+  phases += initial_phase
+  final_phase = phases[:, -1:, 0:1]
+
+  n_harmonics = int(amplitude_envelopes.shape[-1])
+  f_ratios = tf.linspace(1.0, float(n_harmonics), int(n_harmonics))
+  f_ratios = f_ratios[tf.newaxis, tf.newaxis, :]
+  phases = phases * f_ratios
+
+  # Convert to waveforms.
+  wavs = tf.sin(phases)
+  audio = amplitude_envelopes * wavs  # [mb, n_samples, n_sinusoids]
+  audio = tf.reduce_sum(audio, axis=-1)  # [mb, n_samples]
+
+  return audio, final_phase
+
+
 def get_harmonic_frequencies(frequencies: tf.Tensor,
                              n_harmonics: int) -> tf.Tensor:
   """Create integer multiples of the fundamental frequency.
@@ -874,7 +1001,8 @@ def harmonic_synthesis(frequencies: tf.Tensor,
                        harmonic_distribution: Optional[tf.Tensor] = None,
                        n_samples: int = 64000,
                        sample_rate: int = 16000,
-                       amp_resample_method: Text = 'window') -> tf.Tensor:
+                       amp_resample_method: Text = 'window',
+                       use_angular_cumsum: bool = False) -> tf.Tensor:
   """Generate audio from frame-wise monophonic harmonic oscillator bank.
 
   Args:
@@ -891,6 +1019,8 @@ def harmonic_synthesis(frequencies: tf.Tensor,
     n_samples: Total length of output audio. Interpolates and crops to this.
     sample_rate: Sample rate.
     amp_resample_method: Mode with which to resample amplitude envelopes.
+    use_angular_cumsum: Use angular cumulative sum on accumulating phase
+      instead of tf.cumsum. More accurate for inference.
 
   Returns:
     audio: Output audio. Shape [batch_size, n_samples, 1]
@@ -926,10 +1056,64 @@ def harmonic_synthesis(frequencies: tf.Tensor,
   # Synthesize from harmonics [batch_size, n_samples].
   audio = oscillator_bank(frequency_envelopes,
                           amplitude_envelopes,
-                          sample_rate=sample_rate)
+                          sample_rate=sample_rate,
+                          use_angular_cumsum=use_angular_cumsum)
   return audio
 
 
+def streaming_harmonic_synthesis(
+    frequencies: tf.Tensor,
+    amplitudes: tf.Tensor,
+    harmonic_distribution: Optional[tf.Tensor] = None,
+    initial_phase: Optional[tf.Tensor] = None,
+    n_samples: int = 64000,
+    sample_rate: int = 16000,
+    amp_resample_method: Text = 'linear') -> tf.Tensor:
+  """Generate audio from frame-wise monophonic harmonic oscillator bank.
+
+  Args:
+    frequencies: Frame-wise fundamental frequency in Hz. Shape [batch_size,
+      n_frames, 1].
+    amplitudes: Frame-wise oscillator peak amplitude. Shape [batch_size,
+      n_frames, 1].
+    harmonic_distribution: Harmonic amplitude variations, ranged zero to one.
+      Total amplitude of a harmonic is equal to (amplitudes *
+      harmonic_distribution). Shape [batch_size, n_frames, n_harmonics].
+    initial_phase: Starting phase. Shape [batch_size, 1, 1].
+    n_samples: Total length of output audio. Interpolates and crops to this.
+    sample_rate: Sample rate.
+    amp_resample_method: Mode with which to resample amplitude envelopes.
+
+  Returns:
+    audio: Output audio. Shape [batch_size, n_samples, 1]
+  """
+  frequencies = tf_float32(frequencies)
+  amplitudes = tf_float32(amplitudes)
+
+  if harmonic_distribution is not None:
+    # Create harmonic amplitudes [batch_size, n_frames, n_harmonics].
+    harmonic_distribution = tf_float32(harmonic_distribution)
+    # Don't exceed Nyquist.
+    harmonic_distribution = normalize_harmonics(
+        harmonic_distribution, frequencies, sample_rate)
+    harmonic_amplitudes = amplitudes * harmonic_distribution
+  else:
+    harmonic_amplitudes = amplitudes
+
+  # Create sample-wise envelopes.
+  frequencies = resample(frequencies, n_samples)  # cycles/sec
+  amplitude_envelopes = resample(harmonic_amplitudes, n_samples,
+                                 method=amp_resample_method)
+
+  # Synthesize from harmonics [batch_size, n_samples].
+  audio, final_phase = harmonic_oscillator_bank(
+      frequencies,
+      amplitude_envelopes,
+      initial_phase,
+      sample_rate=sample_rate)
+  return audio, final_phase
+
+
 # Wavetable Synthesizer --------------------------------------------------------
 def linear_lookup(phase: tf.Tensor,
                   wavetables: tf.Tensor) -> tf.Tensor:
