@@ -22,6 +22,7 @@ import gin
 import librosa
 import numpy as np
 import tensorflow.compat.v2 as tf
+import tensorflow_probability as tfp
 
 CREPE_SAMPLE_RATE = 16000
 _CREPE_FRAME_SIZE = 1024
@@ -193,7 +194,8 @@ def compute_loudness(audio,
                      n_fft=2048,
                      range_db=LD_RANGE,
                      ref_db=20.7,
-                     use_tf=False):
+                     use_tf=False,
+                     pad_end=True):
   """Perceptual loudness in dB, relative to white noise, amplitude=1.
 
   Function is differentiable if use_tf=True.
@@ -211,6 +213,7 @@ def compute_loudness(audio,
       slight dependence on fft_size due to different granularity of perceptual
       weighting.
     use_tf: Make function differentiable by using tensorflow.
+    pad_end: Add zero padding at end of audio (like `same` convolution).
 
   Returns:
     Loudness in decibels. Shape [batch_size, n_frames] or [n_frames,].
@@ -235,7 +238,7 @@ def compute_loudness(audio,
   hop_size = sample_rate // frame_rate
   overlap = 1 - hop_size / n_fft
   stft_fn = stft if use_tf else stft_np
-  s = stft_fn(audio, frame_size=n_fft, overlap=overlap, pad_end=True)
+  s = stft_fn(audio, frame_size=n_fft, overlap=overlap, pad_end=pad_end)
 
   # Compute power.
   amplitude = lib.abs(s)
@@ -312,18 +315,20 @@ def compute_f0(audio, sample_rate, frame_rate, viterbi=True):
 def compute_rms_energy(audio,
                        sample_rate=16000,
                        frame_rate=250,
-                       frame_size=2048):
+                       frame_size=2048,
+                       pad_end=True):
   """Compute root mean squared energy of audio."""
-  n_samples = audio.shape[0] if len(audio.shape) == 1 else audio.shape[1]
-  n_secs = n_samples / float(sample_rate)  # `n_secs` can have milliseconds
-  expected_len = int(n_secs * frame_rate)
-
   audio = tf_float32(audio)
-
   hop_size = sample_rate // frame_rate
-  audio_frames = tf.signal.frame(audio, frame_size, hop_size, pad_end=True)
+  audio_frames = tf.signal.frame(audio, frame_size, hop_size, pad_end=pad_end)
   rms_energy = tf.reduce_mean(audio_frames**2.0, axis=-1)**0.5
-  return pad_or_trim_to_expected_length(rms_energy, expected_len, use_tf=True)
+  if pad_end:
+    n_samples = audio.shape[0] if len(audio.shape) == 1 else audio.shape[1]
+    n_secs = n_samples / float(sample_rate)  # `n_secs` can have milliseconds
+    expected_len = int(n_secs * frame_rate)
+    return pad_or_trim_to_expected_length(rms_energy, expected_len, use_tf=True)
+  else:
+    return rms_energy
 
 
 def compute_power(audio,
@@ -331,10 +336,12 @@ def compute_power(audio,
                   frame_rate=250,
                   frame_size=1024,
                   range_db=LD_RANGE,
-                  ref_db=20.7):
+                  ref_db=20.7,
+                  pad_end=True):
   """Compute power of audio in dB."""
   # TODO(hanoih@): enable `use_tf` to be True or False like `compute_loudness`
-  rms_energy = compute_rms_energy(audio, sample_rate, frame_rate, frame_size)
+  rms_energy = compute_rms_energy(
+      audio, sample_rate, frame_rate, frame_size, pad_end)
   power_db = amplitude_to_db(rms_energy**2, use_tf=True)
   # Set dynamic range.
   power_db -= ref_db
@@ -405,3 +412,138 @@ def reset_crepe():
   """Reset the global state of CREPE to force model re-building."""
   for k in crepe.core.models:
     crepe.core.models[k] = None
+
+
+class PretrainedCREPE(tf.keras.Model):
+  """A wrapper around a pretrained CREPE model, for pitch prediction.
+
+  Enables predicting pitch and confidence entirely in TF for running in batch
+  on accelerators. For [full,large,small,tiny] crepe models, reads h5 models
+  from installed pip package. Other saved models
+  """
+
+  def __init__(self,
+               model_size_or_path,
+               hop_size=160,
+               **kwargs):
+    super().__init__(**kwargs)
+    self.hop_size = hop_size
+    self.frame_size = 1024
+    self.sample_rate = 16000
+    # Load the crepe model.
+    if model_size_or_path in ['full', 'large', 'small', 'tiny']:
+      self.core_model = crepe.core.build_and_load_model(model_size_or_path)
+    else:
+      self.core_model = tf.keras.models.load_model(model_size_or_path)
+
+    self.model_size_or_path = model_size_or_path
+
+  @classmethod
+  def activations_to_f0_and_confidence(cls, activations, centers=None):
+    """Convert network outputs (activations) to f0 predictions."""
+    cent_mapping = tf.cast(
+        tf.linspace(0, 7180, 360) + 1997.3794084376191, tf.float32)
+
+    # The confidence of voicing activity and the argmax bin.
+    confidence = tf.reduce_max(activations, axis=-1, keepdims=True)
+    if centers is None:
+      centers = tf.math.argmax(activations, axis=-1)
+    centers = tf.cast(centers, tf.int32)
+
+    # Slice the local neighborhood around the argmax bin.
+    start = centers - 4
+    idx_list = tf.range(0, 10)
+    idx_list = start[:, None] + idx_list[None, :]
+
+    # Bound to [0, 359].
+    idx_list = tf.where(idx_list > 0, idx_list, 0)
+    idx_list = tf.where(idx_list < 359, idx_list, 359)
+
+    # Gather and weight activations.
+    weights = tf.gather(activations, idx_list, batch_dims=1)
+    cents = tf.gather(cent_mapping, idx_list, batch_dims=0)
+    f0_cent = tf.reduce_sum(weights * cents, axis=-1) / tf.reduce_sum(
+        weights, axis=-1)
+    f0_hz = 10 * 2**(f0_cent / 1200.)
+
+    return f0_hz, confidence
+
+  def batch_frames(self, audio):
+    """Chop audio into overlapping frames, and push to batch dimension."""
+    if audio.shape[-1] == self.frame_size:
+      return audio
+    else:
+      frames = tf.signal.frame(audio, self.frame_size, self.hop_size)
+      frames = tf.reshape(frames, [-1, self.frame_size])
+      return frames
+
+  def normalize_frames(self, frames):
+    """Normalize each frame -- this is expected by the model."""
+    mu, var = tf.nn.moments(frames, axes=[-1])
+    std = tf.where(tf.abs(var) > 0, tf.sqrt(var), 1e-8)
+    frames -= mu[:, None]
+    frames /= std[:, None]
+    return frames
+
+  def predict_f0_and_confidence(self, audio, viterbi=False):
+    audio = audio[None, :] if len(audio.shape) == 1 else audio
+    batch_size = audio.shape[0]
+
+    frames = self.batch_frames(audio)
+    frames = self.normalize_frames(frames)
+    acts = self.core_model(frames, training=False)
+
+    if viterbi:
+      acts_viterbi = tf.reshape(acts, [batch_size, -1, 360])
+      centers = self.viterbi_decode(acts_viterbi)
+      centers = tf.reshape(centers, [-1])
+    else:
+      centers = None
+
+    f0_hz, confidence = self.activations_to_f0_and_confidence(acts, centers)
+    f0_hz = tf.reshape(f0_hz, [batch_size, -1])
+    confidence = tf.reshape(confidence, [batch_size, -1])
+    return f0_hz, confidence
+
+  def create_hmm(self, num_steps):
+    """Same as the original CREPE viterbi decdoding, but in TF."""
+    # Initial distribution is uniform.
+    initial_distribution = tfp.distributions.Categorical(
+        probs=tf.ones([360]) / 360)
+
+    # Transition probabilities inducing continuous pitch.
+    bins = tf.range(360, dtype=tf.float32)
+    xx, yy = tf.meshgrid(bins, bins)
+    min_transition = 1e-5  # For training stabiity.
+    transition = tf.maximum(12 - abs(xx - yy), min_transition)
+    transition = transition / tf.reduce_sum(transition, axis=1)[:, None]
+    transition = tf.cast(transition, tf.float32)
+    transition_distribution = tfp.distributions.Categorical(
+        probs=transition)
+
+    # Emission probability = fixed probability for self, evenly distribute the
+    # others.
+    self_emission = 0.1
+    emission = (
+        tf.eye(360) * self_emission + tf.ones(shape=(360, 360)) *
+        ((1 - self_emission) / 360.)
+    )
+    emission = tf.cast(emission, tf.float32)[None, ...]
+    observation_distribution = tfp.distributions.Multinomial(
+        total_count=1, probs=emission)
+
+    return tfp.distributions.HiddenMarkovModel(
+        initial_distribution=initial_distribution,
+        transition_distribution=transition_distribution,
+        observation_distribution=observation_distribution,
+        num_steps=num_steps,
+    )
+
+  def viterbi_decode(self, acts):
+    """Adapted from original CREPE viterbi decdoding, but in TF."""
+    num_steps = acts.shape[1]
+    hmm = self.create_hmm(num_steps)
+    centers = hmm.posterior_mode(acts)
+    return centers
+
+
