@@ -53,20 +53,25 @@ class DataProvider(object):
     """A method that returns a tf.data.Dataset."""
     raise NotImplementedError
 
-  def get_batch(self, batch_size, shuffle=True, repeats=-1):
+  def get_batch(self,
+                batch_size,
+                shuffle=True,
+                repeats=-1,
+                drop_remainder=True):
     """Read dataset.
 
     Args:
       batch_size: Size of batch.
       shuffle: Whether to shuffle the examples.
       repeats: Number of times to repeat dataset. -1 for endless repeats.
+      drop_remainder: Whether the last batch should be dropped.
 
     Returns:
       A batched tf.data.Dataset.
     """
     dataset = self.get_dataset(shuffle)
     dataset = dataset.repeat(repeats)
-    dataset = dataset.batch(batch_size, drop_remainder=True)
+    dataset = dataset.batch(batch_size, drop_remainder=drop_remainder)
     dataset = dataset.prefetch(buffer_size=_AUTOTUNE)
     return dataset
 
@@ -120,7 +125,8 @@ class NSynthTfds(TfdsProvider):
                split='train',
                data_dir='gs://tfds-data/datasets',
                sample_rate=16000,
-               frame_rate=250):
+               frame_rate=250,
+               include_note_labels=True):
     """TfdsProvider constructor.
 
     Args:
@@ -130,7 +136,10 @@ class NSynthTfds(TfdsProvider):
         to the public TFDS GCS bucket.
       sample_rate: Sample rate of audio in the dataset.
       frame_rate: Frame rate of features in the dataset.
+      include_note_labels: Return dataset without note-level labels
+        (pitch, instrument).
     """
+    self._include_note_labels = include_note_labels
     if data_dir == 'gs://tfds-data/datasets':
       logging.warning(
           'Using public TFDS GCS bucket to load NSynth. If not running on '
@@ -141,17 +150,9 @@ class NSynthTfds(TfdsProvider):
   def get_dataset(self, shuffle=True):
     """Returns dataset with slight restructuring of feature dictionary."""
     def preprocess_ex(ex):
-      return {
-          'pitch':
-              ex['pitch'],
+      ex_out = {
           'audio':
               ex['audio'],
-          'instrument_source':
-              ex['instrument']['source'],
-          'instrument_family':
-              ex['instrument']['family'],
-          'instrument':
-              ex['instrument']['label'],
           'f0_hz':
               ex['f0']['hz'],
           'f0_confidence':
@@ -159,6 +160,19 @@ class NSynthTfds(TfdsProvider):
           'loudness_db':
               ex['loudness']['db'],
       }
+      if self._include_note_labels:
+        ex_out.update({
+            'pitch':
+                ex['pitch'],
+            'instrument_source':
+                ex['instrument']['source'],
+            'instrument_family':
+                ex['instrument']['family'],
+            'instrument':
+                ex['instrument']['label'],
+        })
+      return ex_out
+
     dataset = super().get_dataset(shuffle)
     dataset = dataset.map(preprocess_ex, num_parallel_calls=_AUTOTUNE)
     return dataset
@@ -237,11 +251,11 @@ class TFRecordProvider(RecordProvider):
 
 
 # ------------------------------------------------------------------------------
-# Zipped DataProvider
+# Multi-dataset DataProviders
 # ------------------------------------------------------------------------------
 @gin.register
-class ZippedProvider(DataProvider):
-  """Combines datasets from two providers with zip."""
+class BaseMultiProvider(DataProvider):
+  """Base class for providers that combine multiple datasets."""
 
   def __init__(self, data_providers, batch_size_ratios=()):
     """Constructor.
@@ -252,7 +266,6 @@ class ZippedProvider(DataProvider):
         These do not need to sum to 1. For example, [2, 1] will produce batches
         with a size ratio of 2 to 1.
     """
-    # Normalize the ratios.
     if batch_size_ratios:
       # Check lengths match.
       if len(batch_size_ratios) != len(data_providers):
@@ -260,8 +273,12 @@ class ZippedProvider(DataProvider):
                          'length as the list of data providers ({}) for varying'
                          'batch sizes.'.format(
                              len(batch_size_ratios), len(data_providers)))
+      # Normalize the ratios.
       total = sum(batch_size_ratios)
       batch_size_ratios = [float(bsr) / total for bsr in batch_size_ratios]
+    else:
+      # Sample evenly from each.
+      batch_size_ratios = [1.0 for _ in data_providers]
 
     # Make sure all sample rates are the same.
     sample_rates = [dp.sample_rate for dp in data_providers]
@@ -277,6 +294,11 @@ class ZippedProvider(DataProvider):
     self._data_providers = data_providers
     self._batch_size_ratios = batch_size_ratios
 
+
+@gin.register
+class ZippedProvider(BaseMultiProvider):
+  """Combines datasets from two providers with zip."""
+
   def get_dataset(self, shuffle=True):
     """Read dataset.
 
@@ -289,13 +311,18 @@ class ZippedProvider(DataProvider):
     datasets = tuple(dp.get_dataset(shuffle) for dp in self._data_providers)
     return tf.data.Dataset.zip(datasets)
 
-  def get_batch(self, batch_size, shuffle=True, repeats=-1):
+  def get_batch(self,
+                batch_size,
+                shuffle=True,
+                repeats=-1,
+                drop_remainder=False):
     """Read dataset.
 
     Args:
       batch_size: Size of batches, can be a list to have varying batch_sizes.
       shuffle: Whether to shuffle the examples.
       repeats: Number of times to repeat dataset. -1 for endless repeats.
+      drop_remainder: Whether the last batch should be dropped.
 
     Returns:
       A batched tf.data.Dataset.
@@ -308,7 +335,7 @@ class ZippedProvider(DataProvider):
       # Varying batch sizes (Integer batch shape for each).
       batch_sizes = [int(batch_size * bsr) for bsr in self._batch_size_ratios]
       datasets = tuple(
-          dp.get_dataset(shuffle).batch(bs, drop_remainder=True)
+          dp.get_dataset(shuffle).batch(bs, drop_remainder=drop_remainder)
           for bs, dp in zip(batch_sizes, self._data_providers))
       dataset = tf.data.Dataset.zip(datasets)
       dataset = dataset.repeat(repeats)
@@ -316,6 +343,24 @@ class ZippedProvider(DataProvider):
       return dataset
 
 
+@gin.register
+class MixedProvider(BaseMultiProvider):
+  """Combines datasets from two providers mixed with sampling."""
+
+  def get_dataset(self, shuffle=True):
+    """Read dataset.
+
+    Args:
+      shuffle: Whether to shuffle the input files.
+
+    Returns:
+      dataset: A tf.data.Dataset mixed from multiple datasets.
+    """
+    datasets = tuple(dp.get_dataset(shuffle) for dp in self._data_providers)
+    return tf.data.experimental.sample_from_datasets(
+        datasets, self._batch_size_ratios)
+
+
 # ------------------------------------------------------------------------------
 # Synthetic Data for InverseSynthesis
 # ------------------------------------------------------------------------------
@@ -366,7 +411,11 @@ class SyntheticNotes(TFRecordProvider):
 class Urmp(TFRecordProvider):
   """Urmp training set."""
 
-  def __init__(self, base_dir, instrument_key='tpt', split='train'):
+  def __init__(self,
+               base_dir,
+               instrument_key='tpt',
+               split='train',
+               suffix=None):
     """URMP dataset for either a specific instrument or all instruments.
 
     Args:
@@ -375,26 +424,42 @@ class Urmp(TFRecordProvider):
         ['all', 'bn', 'cl', 'db', 'fl', 'hn', 'ob', 'sax', 'tba', 'tbn',
         'tpt', 'va', 'vc', 'vn'].
       split: Choices include ['train', 'test'].
+      suffix: Choices include [None, 'batched', 'unbatched'], but broadly
+        applies to any suffix adding to the file pattern.
+        When suffix is not None, will add "_{suffix}" to the file pattern.
+        This option is used in gs://magentadata/datasets/urmp/urmp_20210324.
+        With the "batched" suffix, the dataloader will load tfrecords
+        containing segmented audio samples in 4 seconds. With the "unbatched"
+        suffix, the dataloader will load tfrecords containing unsegmented
+        samples which could be used for learning note sequence in URMP dataset.
+
     """
     self.instrument_key = instrument_key
     self.split = split
     self.base_dir = base_dir
+    self.suffix = '' if suffix is None else '_' + suffix
     super().__init__()
 
   @property
   def default_file_pattern(self):
     if self.instrument_key == 'all':
-      file_pattern = 'all_instruments_{}.tfrecord*'.format(self.split)
+      file_pattern = 'all_instruments_{}{}.tfrecord*'.format(
+          self.split, self.suffix)
     else:
-      file_pattern = 'urmp_{}_solo_ddsp_conditioning_{}.tfrecord*'.format(
-          self.instrument_key, self.split)
+      file_pattern = 'urmp_{}_solo_ddsp_conditioning_{}{}.tfrecord*'.format(
+          self.instrument_key, self.split, self.suffix)
 
     return os.path.join(self.base_dir, file_pattern)
 
 
 @gin.register
 class UrmpMidi(Urmp):
-  """Urmp training set with midi note data."""
+  """Urmp training set with midi note data.
+
+  This class loads the segmented data in tfrecord that contains 4-second audio
+  clips of the URMP dataset. To load tfrecord that contains unsegmented full
+  piece of URMP recording, please use `UrmpMidiUnsegmented` class instead.
+  """
 
   _INSTRUMENTS = ['vn', 'va', 'vc', 'db', 'fl', 'ob', 'cl', 'sax', 'bn', 'tpt',
                   'hn', 'tbn', 'tba']
@@ -408,8 +473,15 @@ class UrmpMidi(Urmp):
         'note_active_frame_indices':
             tf.io.FixedLenFeature([self._feature_length * 128], tf.float32),
         'instrument_id': tf.io.FixedLenFeature([], tf.string),
+        'recording_id': tf.io.FixedLenFeature([], tf.string),
         'power_db':
             tf.io.FixedLenFeature([self._feature_length], dtype=tf.float32),
+        'note_onsets':
+            tf.io.FixedLenFeature([self._feature_length * 128],
+                                  dtype=tf.float32),
+        'note_offsets':
+            tf.io.FixedLenFeature([self._feature_length * 128],
+                                  dtype=tf.float32),
     })
     return base_features
 
@@ -426,6 +498,88 @@ class UrmpMidi(Urmp):
       data['note_active_velocities'] = tf.reshape(
           data['note_active_velocities'], (-1, 128))
       data['instrument_id'] = inst_vocab.lookup(data['instrument_id'])
+      data['midi'] = tf.argmax(data['note_active_frame_indices'], axis=-1)
+      data['f0_hz'] = data['f0_hz'][..., tf.newaxis]
+      data['loudness_db'] = data['loudness_db'][..., tf.newaxis]
+      onsets = tf.reduce_sum(
+          tf.reshape(data['note_onsets'], (-1, 128)), axis=-1)
+      data['onsets'] = tf.cast(onsets > 0, tf.int64)
+      offsets = tf.reduce_sum(
+          tf.reshape(data['note_offsets'], (-1, 128)), axis=-1)
+      data['offsets'] = tf.cast(offsets > 0, tf.int64)
+
+      return data
+
+    ds = super().get_dataset(shuffle)
+    ds = ds.map(_reshape_tensors, num_parallel_calls=_AUTOTUNE)
+    return ds
+
+
+class UrmpMidiUnsegmented(Urmp):
+  """Urmp dataset using unsegmented data.
+
+  Unsegmented here means that the data samples are not segmented to 4-second
+  chunks as in UrmpMidi dataset.
+  """
+
+  _INSTRUMENTS = ['vn', 'va', 'vc', 'db', 'fl', 'ob', 'cl', 'sax', 'bn', 'tpt',
+                  'hn', 'tbn', 'tba']
+
+  @property
+  def features_dict(self):
+    base_features = {
+        'audio':
+            tf.io.VarLenFeature(dtype=tf.float32),
+        'f0_hz':
+            tf.io.VarLenFeature(dtype=tf.float32),
+        'f0_confidence':
+            tf.io.VarLenFeature(dtype=tf.float32),
+        'loudness_db':
+            tf.io.VarLenFeature(dtype=tf.float32),
+    }
+    base_features.update({
+        'note_active_velocities':
+            tf.io.VarLenFeature(tf.float32),
+        'note_active_frame_indices':
+            tf.io.VarLenFeature(tf.float32),
+        'instrument_id': tf.io.FixedLenFeature([], tf.string),
+        'recording_id': tf.io.FixedLenFeature([], tf.string),
+        'power_db':
+            tf.io.VarLenFeature(dtype=tf.float32),
+        'note_onsets':
+            tf.io.VarLenFeature(dtype=tf.float32),
+        'note_offsets':
+            tf.io.VarLenFeature(dtype=tf.float32),
+    })
+    return base_features
+
+  def get_dataset(self, shuffle=True):
+    instrument_ids = range(len(self._INSTRUMENTS))
+    inst_vocab = tf.lookup.StaticHashTable(
+        tf.lookup.KeyValueTensorInitializer(
+            self._INSTRUMENTS, instrument_ids), -1)
+
+    def _reshape_tensors(data):
+      data['audio'] = tf.sparse.to_dense(data['audio'])
+      data['note_active_frame_indices'] = tf.reshape(
+          tf.sparse.to_dense(data['note_active_frame_indices']), (-1, 128))
+      data['note_active_velocities'] = tf.reshape(
+          tf.sparse.to_dense(data['note_active_velocities']), (-1, 128))
+      data['instrument_id'] = inst_vocab.lookup(data['instrument_id'])
+      data['midi'] = tf.argmax(data['note_active_frame_indices'], axis=-1)
+      data['f0_hz'] = tf.sparse.to_dense(data['f0_hz'])[..., tf.newaxis]
+      data['loudness_db'] = tf.sparse.to_dense(data['loudness_db'])[
+          ..., tf.newaxis]
+      # reshape and rename things
+      onsets = tf.reduce_sum(
+          tf.reshape(tf.sparse.to_dense(data['note_onsets']), (-1, 128)),
+          axis=-1)
+      data['onsets'] = tf.cast(onsets > 0, tf.int64)
+      offsets = tf.reduce_sum(
+          tf.reshape(tf.sparse.to_dense(data['note_offsets']), (-1, 128)),
+          axis=-1)
+      data['offsets'] = tf.cast(offsets > 0, tf.int64)
+
       return data
 
     ds = super().get_dataset(shuffle)
